// RotatedEllipseForm.pas
//
// dialog unit to draw rotated ellipse with polygon lines
//
// V1.00 06-02-99, efg, http://www.efg2.com
//
// V1.01 06-23-99, Mr.Pix
//    changed:  faster calculation
//              ellipse drawing procedure extracted
//              window is now resizeable
//    added:    viewing coordination axis
//              viewing ellipse axis
//    problems: flickering on resizing window
//              floating point input not possible with tSpinEdit

unit RotatedEllipseForm;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, StdCtrls, Spin;

type
  TFormRotatedEllipse = class(TForm)
    Image1: TImage;
    Panel1: TPanel;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    SpinEditA: TSpinEdit;
    SpinEditB: TSpinEdit;
    SpinEditLines: TSpinEdit;
    SpinEditAngle: TSpinEdit;
    OKButton: TButton;
    procedure Repaint(Sender: TObject);
    procedure OKClick(Sender: TObject);
  end;

var
  FormRotatedEllipse: TFormRotatedEllipse;

implementation
{$R *.DFM}

type float = double;    // used floating point type

var  dc: tCanvas;

procedure RotatedEllipse (x,y,             // center
                          a,b,             // axis length
                          angle: float;    // rotation angle in radiant
                          lines: integer); // number of lines
var   sinAngle : float;
      cosAngle : float;
      theta    : float;
      xp,yp    : float;
      xr, yr   : integer;
      li       : integer;
begin
  sinAngle := sin (angle);
  cosAngle := cos (angle);
  for li := 0 to lines
  do begin
    theta := (li/lines) * 2*PI;   // angle step
    xp := a * cos(theta);         // ellipse point
    yp := b * sin(theta);

    // rotate ellipse point around center
    xr := Round (x - xp * cosAngle + yp * sinAngle);
    yr := Round (y + xp * sinAngle + yp * cosAngle);
    if li = 0
    then dc.MoveTo(xr, yr)
    else dc.LineTo(xr, yr)
  end;
end;

procedure TFormRotatedEllipse.Repaint(Sender: TObject);

var   a,b      : float;      // axis length
      angle    : float;      // rotation angle
      lines    : integer;    // number of polylines
      x,y      : integer;    // ellipse center
      xr,yr    : integer;    // axis distances
      Bm       : TBitmap;    // help bitmap
begin
  a := SpinEditA.Value;
  b := SpinEditB.Value;
  lines := SpinEditLines.Value;
  angle := SpinEditAngle.Value * PI / 180;
  try
    Bm := TBitmap.Create;
    dc := Bm.Canvas;
    Bm.Width  := Image1.Width;    // set bitmap dimension
    Bm.Height := Image1.Height;
    x := Bm.Width  div 2;         // bitmap center
    y := Bm.Height div 2;

    // draw coordination system
    dc.Pen.Color := clBlack;
    dc.MoveTo(0,y);   dc.LineTo(Bm.Width, y);
    dc.MoveTo(x,0);   dc.LineTo(x, Bm.Height);

    // draw ellipse main axis
    xr := round(a * cos(angle));
    yr := round(a * sin(angle));
    dc.Pen.Color := clBlue;
    dc.MoveTo(x-xr, y+yr);   dc.LineTo(x+xr, y-yr);
    xr := round(b * cos(angle));
    yr := round(b * sin(angle));
    dc.MoveTo(x+yr, y+xr);   dc.LineTo(x-yr, y-xr);

    // draw rotated ellipse
    dc.Pen.Color := clRed;
    RotatedEllipse (x,y, a,b, angle, lines);

    Image1.Picture.Graphic := Bm;
  finally
    Bm.Free;
  end;
end;

procedure TFormRotatedEllipse.OKClick(Sender: TObject);
begin
  Close;
end;

end.
